==> FlightGear-0.7.0.tar.gz: FlightGear-0.7.0/Lib/Voronoi/voronoi.h <==
/*****************************************************************************

 Header:       VORONOI.h	
 Author:       Christian Mayer
 Date started: 28.05.99

 ---------- Copyright (C) 1999  Christian Mayer (vader@t-online.de) ----------

 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
 Foundation; either version 2 of the License, or (at your option) any later
 version.

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 details.

 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA  02111-1307, USA.

 Further information about the GNU General Public License can also be found on
 the world wide web at http://www.gnu.org.

 Structure for the simulated physical properties

FUNCTIONAL DESCRIPTION
------------------------------------------------------------------------------
internal header for the voronoi diagram calculation
based on Steven Fortune 'Sweep2'
NOTE: Voronoi returns a list of points. These points are the border of the
cell. The cell is closed when the last point is the same as the first one. In
infinte cells you might have the effect that the first and/or last points are
inside the convex hull of all points. This is not a bug. This 'wrong' point
lies on the line through the last valid point to infinity and you can use it
w/o problems to calculate the slope of that line.

HISTORY
------------------------------------------------------------------------------
30.05.1999 Christian Mayer	Created
16.06.1999 Durk Talsma		Portability for Linux
20.06.1999 Christian Mayer	added lots of consts
30.06.1999 Christian Mayer	STL portability
*****************************************************************************/
#ifndef VORONOI_H
#define VORONOI_H

#include <Include/compiler.h>

#include "point2d.h"

#include STL_IOSTREAM
#include <vector>
#include <set>
FG_USING_STD(iostream);
FG_USING_STD(vector);
FG_USING_STD(set);
FG_USING_NAMESPACE(std);

//forward definitions
struct VoronoiPoint;
bool operator==(const VoronoiPoint& a, const VoronoiPoint& b);

struct VoronoiPoint : Point2D
{
    bool infinity;

    VoronoiPoint (const float a, const float b) { n[PX2d] = a; n[PY2d] = b; infinity=false;}
    VoronoiPoint (const float a, const float b, const bool i) { n[PX2d] = a; n[PY2d] = b; infinity=i;}
    VoronoiPoint (const bool i) { n[PX2d] = n[PY2d] = 0.0; infinity=i; }
    VoronoiPoint () { n[PX2d] = n[PY2d] = 0.0; infinity=false; }

    friend bool operator==(const VoronoiPoint& a, const VoronoiPoint& b);
};

#ifndef ABS
#   define ABS(x) (((x)>0)?(x):(-(x)))
#endif

inline bool operator==(const VoronoiPoint& a, const VoronoiPoint& b)
{
    return
	(( ABS(a.n[PX2d] - b.n[PX2d]) < fgPoint2_Epsilon ) &&
	 ( ABS(a.n[PY2d] - b.n[PY2d]) < fgPoint2_Epsilon ) && (a.infinity == b.infinity )   );
}

typedef vector<VoronoiPoint> PointList;

struct cell
{
    int ID;
    PointList boundary;
    bool sorted;

    cell(const int IDid, const PointList& b) { ID = IDid; boundary = b; sorted=false;}

    void sort(void)
    {
	if (sorted==true)
	    return; //not implemented yet

	if (boundary.size()<2)
	    return; //what do you want to sort?

	if ((boundary.size()%2)==1)
	    return; //can't sort odd numbers

	PointList::iterator it;
	PointList::iterator sortedIt = boundary.begin();

	VoronoiPoint dummy;

	for (it = boundary.end(); it != boundary.begin();)
	{   //figure out if cell is infinite
	    it--;
	    it--;
	    if ( it->infinity == true )	//I've got the beginning of my list
		break;
	}
	//it is pointing to one point that is my start

	if (sortedIt != it)
	{   //swap
	    dummy = (*it); (*it) = (*sortedIt); (*sortedIt) = dummy;
	    it++; sortedIt++;

	    dummy = (*it); (*it) = (*sortedIt); (*sortedIt) = dummy;
	    it++; sortedIt++;
	}
	else
	{
	    it++; sortedIt++;
	    it++; sortedIt++;
	}

	//now I've got already one piece at the beginning
	
	for (;sortedIt != boundary.end();)
	{
	    sortedIt--; //point to last entry

	    for (it = boundary.end(),it--; it != sortedIt;)
	    {   //find next in the sequence
		it--;
		if ((*it)==(*sortedIt))
		{   
		    it--;
		    break;	//I've found the matching piece
		}
		it--;
	    }

	    if (sortedIt != it)
	    {   //swap
		sortedIt++; it++;

		dummy = (*it); (*it) = (*sortedIt); (*sortedIt) = dummy;
		it++; sortedIt++;
		
		dummy = (*it); (*it) = (*sortedIt); (*sortedIt) = dummy;
		sortedIt++;
	    }
	    else
	    {
		sortedIt++; sortedIt++; sortedIt++;
	    }
	    
	}
	
    }

    void strip(void)
    {
	PointList::iterator it;
	PointList::iterator it2 = boundary.end();

	it2--; it=it2; it--;

	for (; it != boundary.begin(); it--)
	{
	    if ((*it)==(*it2))
	    {
		boundary.erase(it2);
	    }
	    it2--;
	}
    }
};

inline bool operator<(const cell& a, const cell& b)
{
    return (a.ID < b.ID);
}

inline ostream& operator<<(ostream& out, cell& c)
{
    out << "Cell ID: " << c.ID << "\nBoundaray: ";
    
    for (PointList::iterator it = c.boundary.begin(); it != c.boundary.end(); it++)
    {
	if (it != c.boundary.begin())
	    out << "-";

	if ((it->infinity)==true)
	    out << "[" << (*it) << "]";
	else
	    out << "(" << (*it) << ")";
    }
    
    return out << "\n";
}

typedef vector<cell> cellList;

extern cellList cl;

#endif /*VORONOI_H*/

==> FlightGear-0.7.1.tar.gz: FlightGear-0.7.1/Lib/Voronoi/voronoi.h <==
/*****************************************************************************

 Header:       VORONOI.h	
 Author:       Christian Mayer
 Date started: 28.05.99

 ---------- Copyright (C) 1999  Christian Mayer (vader@t-online.de) ----------

 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
 Foundation; either version 2 of the License, or (at your option) any later
 version.

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 details.

 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA  02111-1307, USA.

 Further information about the GNU General Public License can also be found on
 the world wide web at http://www.gnu.org.

 Structure for the simulated physical properties

FUNCTIONAL DESCRIPTION
------------------------------------------------------------------------------
internal header for the voronoi diagram calculation
based on Steven Fortune 'Sweep2'
NOTE: Voronoi returns a list of points. These points are the border of the
cell. The cell is closed when the last point is the same as the first one. In
infinte cells you might have the effect that the first and/or last points are
inside the convex hull of all points. This is not a bug. This 'wrong' point
lies on the line through the last valid point to infinity and you can use it
w/o problems to calculate the slope of that line.

HISTORY
------------------------------------------------------------------------------
30.05.1999 Christian Mayer	Created
16.06.1999 Durk Talsma		Portability for Linux
20.06.1999 Christian Mayer	added lots of consts
30.06.1999 Christian Mayer	STL portability
19.10.1999 Norman Vine		reducing most of the warnings. Thanks!
19.10.1999 Christian Mayer	change to use PLIB's sg instead of Point[2/3]D
				and lots of wee code cleaning
*****************************************************************************/
#ifndef VORONOI_H
#define VORONOI_H

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <Include/compiler.h>

#ifdef HAVE_WINDOWS_H
#  include <windows.h>
#endif

#include "sg.h"

#include STL_IOSTREAM
#include <vector>
#include <set>
// FG_USING_STD(iostream);
FG_USING_STD(vector);
FG_USING_STD(set);
FG_USING_NAMESPACE(std);

//forward definitions
struct VoronoiPoint;
bool operator==(const VoronoiPoint& a, const VoronoiPoint& b);

struct VoronoiPoint 
{
    sgVec2 p;
    bool infinity;

    VoronoiPoint (const float a, const float b) { p[0] = a; p[1] = b; infinity=false;}
    VoronoiPoint (const float a, const float b, const bool i) { p[0] = a; p[1] = b; infinity=i;}
    VoronoiPoint (const bool i) { p[0] = p[1] = 0.0; infinity=i; }
    VoronoiPoint () { p[0] = p[1] = 0.0; infinity=false; }

    friend bool operator==(const VoronoiPoint& a, const VoronoiPoint& b);
};

#ifndef ABS
#   define ABS(x) (((x)>0)?(x):(-(x)))
#endif

#define VoronoiPointTol 1e-7

inline bool operator==(const VoronoiPoint& a, const VoronoiPoint& b)
{
    return ( sgCompareVec2(a.p, b.p, VoronoiPointTol) && (a.infinity == b.infinity ) );
}

typedef vector<VoronoiPoint> PointList;

struct cell
{
    int ID;
    PointList boundary;
    bool sorted;

    cell(const int IDid, const PointList& b) { ID = IDid; boundary = b; sorted=false;}

    void sort(void)
    {
	if (sorted==true)
	    return; //not implemented yet

	if (boundary.size()<2)
	    return; //what do you want to sort?

	if ((boundary.size()%2)==1)
	    return; //can't sort odd numbers

	PointList::iterator it;
	PointList::iterator sortedIt = boundary.begin();

	VoronoiPoint dummy;

	for (it = boundary.end(); it != boundary.begin();)
	{   //figure out if cell is infinite
	    it--;
	    it--;
	    if ( it->infinity == true )	//I've got the beginning of my list
		break;
	}
	//it is pointing to one point that is my start

	if (sortedIt != it)
	{   //swap
	    dummy = (*it); (*it) = (*sortedIt); (*sortedIt) = dummy;
	    it++; sortedIt++;

	    dummy = (*it); (*it) = (*sortedIt); (*sortedIt) = dummy;
	    it++; sortedIt++;
	}
	else
	{
	    it++; sortedIt++;
	    it++; sortedIt++;
	}

	//now I've got already one piece at the beginning
	
	for (;sortedIt != boundary.end();)
	{
	    sortedIt--; //point to last entry

	    for (it = boundary.end(),it--; it != sortedIt;)
	    {   //find next in the sequence
		it--;
		if ((*it)==(*sortedIt))
		{   
		    it--;
		    break;	//I've found the matching piece
		}
		it--;
	    }

	    if (sortedIt != it)
	    {   //swap
		sortedIt++; it++;

		dummy = (*it); (*it) = (*sortedIt); (*sortedIt) = dummy;
		it++; sortedIt++;
		
		dummy = (*it); (*it) = (*sortedIt); (*sortedIt) = dummy;
		sortedIt++;
	    }
	    else
	    {
		sortedIt++; sortedIt++; sortedIt++;
	    }
	    
	}
	
    }

    void strip(void)
    {
	PointList::iterator it;
	PointList::iterator it2 = boundary.end();

	it2--; it=it2; it--;

	for (; it != boundary.begin(); it--)
	{
	    if ((*it)==(*it2))
	    {
		boundary.erase(it2);
	    }
	    it2--;
	}
    }
};

inline bool operator<(const cell& a, const cell& b)
{
    return (a.ID < b.ID);
}

inline ostream& operator<<(ostream& out, cell& c)
{
    out << "Cell ID: " << c.ID << "\nBoundaray: ";
    
    for (PointList::iterator it = c.boundary.begin(); it != c.boundary.end(); it++)
    {
	if (it != c.boundary.begin())
	    out << "-";

	if ((it->infinity)==true)
	    out << "[" << it->p[0] << "," << it->p[1] << "]";
	else
	    out << "(" << it->p[0] << "," << it->p[1] << ")";
    }
    
    return out << "\n";
}

typedef vector<cell> cellList;

extern cellList cl;

#endif /*VORONOI_H*/
